@@ -25,18 +25,8 @@
 
 #define FP_COMPONENT "upeksonly"
 
-#include <errno.h>
-#include <string.h>
-
-#include <glib.h>
-#include <libusb.h>
-
-#include <fp_internal.h>
-
-#include <assembling.h>
-
+#include "drivers_api.h"
 #include "upeksonly.h"
-#include "driver_ids.h"
 
 #define CTRL_TIMEOUT	1000
 #define NUM_BULK_TRANSFERS 24
@@ -186,7 +176,7 @@ static void free_img_transfers(struct sonly_dev *sdev)
 
 static void last_transfer_killed(struct fp_img_dev *dev)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	switch (sdev->killing_transfers) {
 	case ABORT_SSM:
 		fp_dbg("abort ssm error %d", sdev->kill_status_code);
@@ -207,7 +197,7 @@ static void last_transfer_killed(struct fp_img_dev *dev)
 
 static void cancel_img_transfers(struct fp_img_dev *dev)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	int i;
 
 	if (sdev->num_flying == 0) {
@@ -234,7 +224,7 @@ static gboolean is_capturing(struct sonly_dev *sdev)
 
 static void handoff_img(struct fp_img_dev *dev)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	struct fp_img *img;
 
 	GSList *elem = sdev->rows;
@@ -246,7 +236,7 @@ static void handoff_img(struct fp_img_dev *dev)
 
 	sdev->rows = g_slist_reverse(sdev->rows);
 
-	fp_dbg("%d rows", sdev->num_rows);
+	fp_dbg("%lu rows", sdev->num_rows);
 	img = fpi_assemble_lines(&assembling_ctx, sdev->rows, sdev->num_rows);
 
 	g_slist_free_full(sdev->rows, g_free);
@@ -262,7 +252,7 @@ static void handoff_img(struct fp_img_dev *dev)
 
 static void row_complete(struct fp_img_dev *dev)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	sdev->rowbuf_offset = -1;
 
 	if (sdev->num_rows > 0) {
@@ -312,7 +302,7 @@ static void row_complete(struct fp_img_dev *dev)
 			 */
 			if (sdev->num_blank > FINGER_REMOVED_THRESHOLD) {
 				sdev->finger_state = FINGER_REMOVED;
-				fp_dbg("detected finger removal. Blank rows: %d, Full rows: %d", sdev->num_blank, sdev->num_rows);
+				fp_dbg("detected finger removal. Blank rows: %d, Full rows: %lu", sdev->num_blank, sdev->num_rows);
 				handoff_img(dev);
 				return;
 			}
@@ -350,7 +340,7 @@ static void row_complete(struct fp_img_dev *dev)
 /* add data to row buffer */
 static void add_to_rowbuf(struct fp_img_dev *dev, unsigned char *data, int size)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
 	memcpy(sdev->rowbuf + sdev->rowbuf_offset, data, size);
 	sdev->rowbuf_offset += size;
@@ -384,7 +374,7 @@ static int rowbuf_remaining(struct sonly_dev *sdev)
 
 static void handle_packet(struct fp_img_dev *dev, unsigned char *data)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	uint16_t seqnum = data[0] << 8 | data[1];
 	int abs_base_addr;
 	int for_rowbuf;
@@ -480,7 +470,7 @@ static void img_data_cb(struct libusb_transfer *transfer)
 {
 	struct img_transfer_data *idata = transfer->user_data;
 	struct fp_img_dev *dev = idata->dev;
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	int i;
 
 	idata->flying = FALSE;
@@ -589,6 +579,7 @@ static void sm_write_regs(struct fpi_ssm *ssm,
 {
 	struct write_regs_data *wrdata = g_malloc(sizeof(*wrdata));
 	unsigned char *data;
+	struct fp_dev *dev;
 
 	wrdata->transfer = libusb_alloc_transfer(0);
 	if (!wrdata->transfer) {
@@ -599,7 +590,9 @@ static void sm_write_regs(struct fpi_ssm *ssm,
 
 	data = g_malloc(LIBUSB_CONTROL_SETUP_SIZE + 1);
 	libusb_fill_control_setup(data, 0x40, 0x0c, 0, 0, 1);
-	libusb_fill_control_transfer(wrdata->transfer, ssm->dev->udev, data,
+	dev = fpi_ssm_get_dev(ssm);
+	libusb_fill_control_transfer(wrdata->transfer,
+		fpi_dev_get_usb_dev(dev), data,
 		write_regs_cb, wrdata, CTRL_TIMEOUT);
 	wrdata->transfer->flags = LIBUSB_TRANSFER_SHORT_NOT_OK;
 
@@ -623,7 +616,7 @@ static void sm_write_reg_cb(struct libusb_transfer *transfer)
 
 static void sm_write_reg(struct fpi_ssm *ssm, uint8_t reg, uint8_t value)
 {
-	struct fp_img_dev *dev = ssm->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
 	struct libusb_transfer *transfer = libusb_alloc_transfer(0);
 	unsigned char *data;
 	int r;
@@ -636,7 +629,8 @@ static void sm_write_reg(struct fpi_ssm *ssm, uint8_t reg, uint8_t value)
 	fp_dbg("set %02x=%02x", reg, value);
 	data = g_malloc(LIBUSB_CONTROL_SETUP_SIZE + 1);
 	libusb_fill_control_setup(data, 0x40, 0x0c, 0, reg, 1);
-	libusb_fill_control_transfer(transfer, dev->udev, data, sm_write_reg_cb,
+	libusb_fill_control_transfer(transfer, fpi_imgdev_get_usb_dev(dev),
+		data, sm_write_reg_cb,
 		ssm, CTRL_TIMEOUT);
 
 	data[LIBUSB_CONTROL_SETUP_SIZE] = value;
@@ -654,8 +648,8 @@ static void sm_write_reg(struct fpi_ssm *ssm, uint8_t reg, uint8_t value)
 static void sm_read_reg_cb(struct libusb_transfer *transfer)
 {
 	struct fpi_ssm *ssm = transfer->user_data;
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
 	if (transfer->status != LIBUSB_TRANSFER_COMPLETED) {
 		fpi_ssm_mark_aborted(ssm, -EIO);
@@ -670,7 +664,7 @@ static void sm_read_reg_cb(struct libusb_transfer *transfer)
 
 static void sm_read_reg(struct fpi_ssm *ssm, uint8_t reg)
 {
-	struct fp_img_dev *dev = ssm->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
 	struct libusb_transfer *transfer = libusb_alloc_transfer(0);
 	unsigned char *data;
 	int r;
@@ -683,7 +677,8 @@ static void sm_read_reg(struct fpi_ssm *ssm, uint8_t reg)
 	fp_dbg("read reg %02x", reg);
 	data = g_malloc(LIBUSB_CONTROL_SETUP_SIZE + 8);
 	libusb_fill_control_setup(data, 0xc0, 0x0c, 0, reg, 8);
-	libusb_fill_control_transfer(transfer, dev->udev, data, sm_read_reg_cb,
+	libusb_fill_control_transfer(transfer, fpi_imgdev_get_usb_dev(dev),
+		data, sm_read_reg_cb,
 		ssm, CTRL_TIMEOUT);
 	transfer->flags = LIBUSB_TRANSFER_SHORT_NOT_OK |
 		LIBUSB_TRANSFER_FREE_TRANSFER;
@@ -699,8 +694,8 @@ static void sm_read_reg(struct fpi_ssm *ssm, uint8_t reg)
 static void sm_await_intr_cb(struct libusb_transfer *transfer)
 {
 	struct fpi_ssm *ssm = transfer->user_data;
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
 	if (transfer->status != LIBUSB_TRANSFER_COMPLETED) {
 		g_free(transfer->buffer);
@@ -720,7 +715,7 @@ static void sm_await_intr_cb(struct libusb_transfer *transfer)
 
 static void sm_await_intr(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
 	struct libusb_transfer *transfer = libusb_alloc_transfer(0);
 	unsigned char *data;
 	int r;
@@ -730,9 +725,10 @@ static void sm_await_intr(struct fpi_ssm *ssm)
 		return;
 	}
 
-	fp_dbg("");
+	G_DEBUG_HERE();
 	data = g_malloc(4);
-	libusb_fill_interrupt_transfer(transfer, dev->udev, 0x83, data, 4,
+	libusb_fill_interrupt_transfer(transfer, fpi_imgdev_get_usb_dev(dev),
+		0x83, data, 4,
 		sm_await_intr_cb, ssm, 0);
 	transfer->flags = LIBUSB_TRANSFER_SHORT_NOT_OK |
 		LIBUSB_TRANSFER_FREE_TRANSFER;
@@ -769,10 +765,10 @@ enum awfsm_1000_states {
 
 static void awfsm_2016_run_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case AWFSM_2016_WRITEV_1:
 		sm_write_regs(ssm, awfsm_2016_writev_1, G_N_ELEMENTS(awfsm_2016_writev_1));
 		break;
@@ -816,7 +812,7 @@ static void awfsm_2016_run_state(struct fpi_ssm *ssm)
 
 static void awfsm_1000_run_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case AWFSM_1000_WRITEV_1:
 		sm_write_regs(ssm, awfsm_1000_writev_1, G_N_ELEMENTS(awfsm_1000_writev_1));
 		break;
@@ -857,8 +853,8 @@ enum capsm_1001_states {
 
 static void capsm_fire_bulk(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	int i;
 	for (i = 0; i < NUM_BULK_TRANSFERS; i++) {
 		int r = libusb_submit_transfer(sdev->img_transfer[i]);
@@ -887,10 +883,10 @@ static void capsm_fire_bulk(struct fpi_ssm *ssm)
 
 static void capsm_2016_run_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case CAPSM_2016_INIT:
 		sdev->rowbuf_offset = -1;
 		sdev->num_rows = 0;
@@ -919,10 +915,10 @@ static void capsm_2016_run_state(struct fpi_ssm *ssm)
 
 static void capsm_1000_run_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case CAPSM_1000_INIT:
 		sdev->rowbuf_offset = -1;
 		sdev->num_rows = 0;
@@ -945,10 +941,10 @@ static void capsm_1000_run_state(struct fpi_ssm *ssm)
 
 static void capsm_1001_run_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case CAPSM_1001_INIT:
 		sdev->rowbuf_offset = -1;
 		sdev->num_rows = 0;
@@ -1000,7 +996,7 @@ enum deinitsm_1001_states {
 
 static void deinitsm_2016_run_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case DEINITSM_2016_WRITEV:
 		sm_write_regs(ssm, deinitsm_2016_writev, G_N_ELEMENTS(deinitsm_2016_writev));
 		break;
@@ -1009,7 +1005,7 @@ static void deinitsm_2016_run_state(struct fpi_ssm *ssm)
 
 static void deinitsm_1000_run_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case DEINITSM_1000_WRITEV:
 		sm_write_regs(ssm, deinitsm_1000_writev, G_N_ELEMENTS(deinitsm_1000_writev));
 		break;
@@ -1018,7 +1014,7 @@ static void deinitsm_1000_run_state(struct fpi_ssm *ssm)
 
 static void deinitsm_1001_run_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case DEINITSM_1001_WRITEV:
 		sm_write_regs(ssm, deinitsm_1001_writev, G_N_ELEMENTS(deinitsm_1001_writev));
 		break;
@@ -1054,10 +1050,10 @@ enum initsm_1001_states {
 
 static void initsm_2016_run_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case INITSM_2016_WRITEV_1:
 		sm_write_regs(ssm, initsm_2016_writev_1, G_N_ELEMENTS(initsm_2016_writev_1));
 		break;
@@ -1084,7 +1080,7 @@ static void initsm_2016_run_state(struct fpi_ssm *ssm)
 
 static void initsm_1000_run_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case INITSM_1000_WRITEV_1:
 		sm_write_regs(ssm, initsm_1000_writev_1, G_N_ELEMENTS(initsm_1000_writev_1));
 		break;
@@ -1093,7 +1089,7 @@ static void initsm_1000_run_state(struct fpi_ssm *ssm)
 
 static void initsm_1001_run_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case INITSM_1001_WRITEV_1:
 		sm_write_regs(ssm, initsm_1001_writev_1, G_N_ELEMENTS(initsm_1001_writev_1));
 		break;
@@ -1126,10 +1122,10 @@ enum loopsm_states {
 
 static void loopsm_run_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	switch (ssm->cur_state) {
+	switch (fpi_ssm_get_cur_state(ssm)) {
 	case LOOPSM_RUN_AWFSM: ;
 		switch (sdev->dev_model) {
 		case UPEKSONLY_1001:
@@ -1146,15 +1142,15 @@ static void loopsm_run_state(struct fpi_ssm *ssm)
 				struct fpi_ssm *awfsm = NULL;
 				switch (sdev->dev_model) {
 				case UPEKSONLY_2016:
-					awfsm = fpi_ssm_new(dev->dev, awfsm_2016_run_state,
+					awfsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), awfsm_2016_run_state,
 						AWFSM_2016_NUM_STATES);
 					break;
 				case UPEKSONLY_1000:
-					awfsm = fpi_ssm_new(dev->dev, awfsm_1000_run_state,
+					awfsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), awfsm_1000_run_state,
 						AWFSM_1000_NUM_STATES);
 					break;
 				}
-				awfsm->priv = dev;
+				fpi_ssm_set_user_data(awfsm, dev);
 				fpi_ssm_start_subsm(ssm, awfsm);
 			}
 			break;
@@ -1174,19 +1170,19 @@ static void loopsm_run_state(struct fpi_ssm *ssm)
 		struct fpi_ssm *capsm = NULL;
 		switch (sdev->dev_model) {
 		case UPEKSONLY_2016:
-			capsm = fpi_ssm_new(dev->dev, capsm_2016_run_state,
+			capsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), capsm_2016_run_state,
 				CAPSM_2016_NUM_STATES);
 			break;
 		case UPEKSONLY_1000:
-			capsm = fpi_ssm_new(dev->dev, capsm_1000_run_state,
+			capsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), capsm_1000_run_state,
 				CAPSM_1000_NUM_STATES);
 			break;
 		case UPEKSONLY_1001:
-			capsm = fpi_ssm_new(dev->dev, capsm_1001_run_state,
+			capsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), capsm_1001_run_state,
 				CAPSM_1001_NUM_STATES);
 			break;
 		}
-		capsm->priv = dev;
+		fpi_ssm_set_user_data(capsm, dev);
 		fpi_ssm_start_subsm(ssm, capsm);
 		break;
 	case LOOPSM_CAPTURE:
@@ -1195,20 +1191,20 @@ static void loopsm_run_state(struct fpi_ssm *ssm)
 		struct fpi_ssm *deinitsm = NULL;
 		switch (sdev->dev_model) {
 		case UPEKSONLY_2016:
-			deinitsm = fpi_ssm_new(dev->dev, deinitsm_2016_run_state,
+			deinitsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), deinitsm_2016_run_state,
 				DEINITSM_2016_NUM_STATES);
 			break;
 		case UPEKSONLY_1000:
-			deinitsm = fpi_ssm_new(dev->dev, deinitsm_1000_run_state,
+			deinitsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), deinitsm_1000_run_state,
 				DEINITSM_1000_NUM_STATES);
 			break;
 		case UPEKSONLY_1001:
-			deinitsm = fpi_ssm_new(dev->dev, deinitsm_1001_run_state,
+			deinitsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), deinitsm_1001_run_state,
 				DEINITSM_1001_NUM_STATES);
 			break;
 		}
 		sdev->capturing = FALSE;
-		deinitsm->priv = dev;
+		fpi_ssm_set_user_data(deinitsm, dev);
 		fpi_ssm_start_subsm(ssm, deinitsm);
 		break;
 	case LOOPSM_FINAL:
@@ -1222,9 +1218,9 @@ static void loopsm_run_state(struct fpi_ssm *ssm)
 
 static void deactivate_done(struct fp_img_dev *dev)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
-	fp_dbg("");
+	G_DEBUG_HERE();
 	free_img_transfers(sdev);
 	g_free(sdev->rowbuf);
 	sdev->rowbuf = NULL;
@@ -1239,7 +1235,7 @@ static void deactivate_done(struct fp_img_dev *dev)
 
 static void dev_deactivate(struct fp_img_dev *dev)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 
 	if (!sdev->capturing) {
 		deactivate_done(dev);
@@ -1254,9 +1250,9 @@ static void dev_deactivate(struct fp_img_dev *dev)
 
 static void loopsm_complete(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
-	int r = ssm->error;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
+	int r = fpi_ssm_get_error(ssm);
 
 	fpi_ssm_free(ssm);
 
@@ -1273,23 +1269,23 @@ static void loopsm_complete(struct fpi_ssm *ssm)
 
 static void initsm_complete(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct sonly_dev *sdev = dev->priv;
-	int r = ssm->error;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
+	int r = fpi_ssm_get_error(ssm);
 
 	fpi_ssm_free(ssm);
 	fpi_imgdev_activate_complete(dev, r);
 	if (r != 0)
 		return;
 
-	sdev->loopsm = fpi_ssm_new(dev->dev, loopsm_run_state, LOOPSM_NUM_STATES);
-	sdev->loopsm->priv = dev;
+	sdev->loopsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), loopsm_run_state, LOOPSM_NUM_STATES);
+	fpi_ssm_set_user_data(sdev->loopsm, dev);
 	fpi_ssm_start(sdev->loopsm, loopsm_complete);
 }
 
 static int dev_activate(struct fp_img_dev *dev, enum fp_imgdev_state state)
 {
-	struct sonly_dev *sdev = dev->priv;
+	struct sonly_dev *sdev = fpi_imgdev_get_user_data(dev);
 	struct fpi_ssm *ssm = NULL;
 	int i;
 
@@ -1311,71 +1307,35 @@ static int dev_activate(struct fp_img_dev *dev, enum fp_imgdev_state state)
 		sdev->img_transfer_data[i].idx = i;
 		sdev->img_transfer_data[i].dev = dev;
 		data = g_malloc(4096);
-		libusb_fill_bulk_transfer(sdev->img_transfer[i], dev->udev, 0x81, data,
+		libusb_fill_bulk_transfer(sdev->img_transfer[i], fpi_imgdev_get_usb_dev(dev),
+			0x81, data,
 			4096, img_data_cb, &sdev->img_transfer_data[i], 0);
 	}
 
 	switch (sdev->dev_model) {
 	case UPEKSONLY_2016:
-		ssm = fpi_ssm_new(dev->dev, initsm_2016_run_state, INITSM_2016_NUM_STATES);
+		ssm = fpi_ssm_new(fpi_imgdev_get_dev(dev), initsm_2016_run_state, INITSM_2016_NUM_STATES);
 		break;
 	case UPEKSONLY_1000:
-		ssm = fpi_ssm_new(dev->dev, initsm_1000_run_state, INITSM_1000_NUM_STATES);
+		ssm = fpi_ssm_new(fpi_imgdev_get_dev(dev), initsm_1000_run_state, INITSM_1000_NUM_STATES);
 		break;
 	case UPEKSONLY_1001:
-		ssm = fpi_ssm_new(dev->dev, initsm_1001_run_state, INITSM_1001_NUM_STATES);
+		ssm = fpi_ssm_new(fpi_imgdev_get_dev(dev), initsm_1001_run_state, INITSM_1001_NUM_STATES);
 		break;
 	}
-	ssm->priv = dev;
+	fpi_ssm_set_user_data(ssm, dev);
 	fpi_ssm_start(ssm, initsm_complete);
 	return 0;
 }
 
-static int dev_init(struct fp_img_dev *dev, unsigned long driver_data)
-{
-	int r;
-	struct sonly_dev *sdev;
-
-	r = libusb_set_configuration(dev->udev, 1);
-	if (r < 0) {
-		fp_err("could not set configuration 1");
-		return r;
-	}
-
-	r = libusb_claim_interface(dev->udev, 0);
-	if (r < 0) {
-		fp_err("could not claim interface 0: %s", libusb_error_name(r));
-		return r;
-	}
-
-	sdev = dev->priv = g_malloc0(sizeof(struct sonly_dev));
-	sdev->dev_model = (int)driver_data;
-	switch (driver_data) {
-	case UPEKSONLY_1000:
-		sdev->img_width = IMG_WIDTH_1000;
-		upeksonly_driver.img_width = IMG_WIDTH_1000;
-		assembling_ctx.line_width = IMG_WIDTH_1000;
-		break;
-	case UPEKSONLY_1001:
-		sdev->img_width = IMG_WIDTH_1001;
-		upeksonly_driver.img_width = IMG_WIDTH_1001;
-		upeksonly_driver.bz3_threshold = 25;
-		assembling_ctx.line_width = IMG_WIDTH_1001;
-		break;
-	case UPEKSONLY_2016:
-		sdev->img_width = IMG_WIDTH_2016;
-		upeksonly_driver.img_width = IMG_WIDTH_2016;
-		assembling_ctx.line_width = IMG_WIDTH_2016;
-		break;
-	}
-	fpi_imgdev_open_complete(dev, 0);
-	return 0;
-}
+static int dev_init(struct fp_img_dev *dev, unsigned long driver_data);
 
 static void dev_deinit(struct fp_img_dev *dev)
 {
-	g_free(dev->priv);
-	libusb_release_interface(dev->udev, 0);
+	void *user_data;
+	user_data = fpi_imgdev_get_user_data(dev);
+	g_free(user_data);
+	libusb_release_interface(fpi_imgdev_get_usb_dev(dev), 0);
 	fpi_imgdev_close_complete(dev);
 }
 
@@ -1422,3 +1382,46 @@ struct fp_img_driver upeksonly_driver = {
 	.deactivate = dev_deactivate,
 };
 
+static int dev_init(struct fp_img_dev *dev, unsigned long driver_data)
+{
+	int r;
+	struct sonly_dev *sdev;
+
+	r = libusb_set_configuration(fpi_imgdev_get_usb_dev(dev), 1);
+	if (r < 0) {
+		fp_err("could not set configuration 1");
+		return r;
+	}
+
+	r = libusb_claim_interface(fpi_imgdev_get_usb_dev(dev), 0);
+	if (r < 0) {
+		fp_err("could not claim interface 0: %s", libusb_error_name(r));
+		return r;
+	}
+
+	sdev = g_malloc0(sizeof(struct sonly_dev));
+	fpi_imgdev_set_user_data(dev, sdev);
+	sdev->dev_model = (int)driver_data;
+	switch (driver_data) {
+	case UPEKSONLY_1000:
+		sdev->img_width = IMG_WIDTH_1000;
+		upeksonly_driver.img_width = IMG_WIDTH_1000;
+		assembling_ctx.line_width = IMG_WIDTH_1000;
+		break;
+	case UPEKSONLY_1001:
+		sdev->img_width = IMG_WIDTH_1001;
+		upeksonly_driver.img_width = IMG_WIDTH_1001;
+		upeksonly_driver.bz3_threshold = 25;
+		assembling_ctx.line_width = IMG_WIDTH_1001;
+		break;
+	case UPEKSONLY_2016:
+		sdev->img_width = IMG_WIDTH_2016;
+		upeksonly_driver.img_width = IMG_WIDTH_2016;
+		assembling_ctx.line_width = IMG_WIDTH_2016;
+		break;
+	}
+	fpi_imgdev_open_complete(dev, 0);
+	return 0;
+}
+
+
