@@ -24,9 +24,15 @@
 #include "fp_internal.h"
 
 static const struct usb_id whitelist_id_table[] = {
-    { .vendor = 0x08ff, .product = 0x2810 },
-    /* https://bugzilla.redhat.com/show_bug.cgi?id=1173367 */
-    { .vendor = 0x138a, .product = 0x0017 },
+    /* Unsupported (for now) Elantech finger print readers */
+    { .vendor = 0x04f3, .product = 0x0c03 },
+    { .vendor = 0x04f3, .product = 0x0c16 },
+    { .vendor = 0x04f3, .product = 0x0c26 },
+    /* Unsupported (for now) Validity Sensors finger print readers */
+    { .vendor = 0x138a, .product = 0x0090 }, /* Found on e.g. Lenovo T460s */
+    { .vendor = 0x138a, .product = 0x0091 },
+    { .vendor = 0x138a, .product = 0x0094 },
+    { .vendor = 0x138a, .product = 0x0097 }, /* Found on e.g. Lenovo T470s */
     { 0, 0, 0, },
 };
 
